@@ -10,12 +10,12 @@ class Test_yaml_set():
     def test_no_options(self, script_runner):
         result = script_runner.run(self.command)
         assert not result.success, result.stderr
-        assert "the following arguments are required: -g/--change, YAML_FILE" in result.stderr
+        assert "the following arguments are required: -g/--change" in result.stderr
 
     def test_no_input_file(self, script_runner):
-        result = script_runner.run(self.command, "--change='/test'")
+        result = script_runner.run(self.command, "--nostdin", "--change='/test'")
         assert not result.success, result.stderr
-        assert "the following arguments are required: YAML_FILE" in result.stderr
+        assert "There must be a YAML_FILE or STDIN document" in result.stderr
 
     def test_no_input_param(self, script_runner):
         result = script_runner.run(self.command, "--change='/test'", "no-such-file")
@@ -32,6 +32,11 @@ class Test_yaml_set():
         assert not result.success, result.stderr
         assert "Impossible to save the old value to the same YAML Path as the new value!" in result.stderr
 
+    def test_insufficient_randomness(self, script_runner):
+        result = script_runner.run(self.command, "--change='/test'", "--random=1", "--random-from=A", "no-such-file")
+        assert not result.success, result.stderr
+        assert "The pool of random CHARS must have at least 2 characters" in result.stderr
+
     def test_bad_privatekey(self, script_runner):
         result = script_runner.run(self.command, "--change='/test'", "--random=1", "--privatekey=no-such-file", "no-such-file")
         assert not result.success, result.stderr
@@ -42,6 +47,37 @@ class Test_yaml_set():
         assert not result.success, result.stderr
         assert "EYAML public key is not a readable file" in result.stderr
 
+    def test_no_dual_stdin(self, script_runner):
+        result = script_runner.run(self.command, "--change='/test'", "--stdin", "-")
+        assert not result.success, result.stderr
+        assert "Impossible to read both document and replacement value from STDIN" in result.stderr
+
+    def test_no_backup_stdin(self, script_runner):
+        result = script_runner.run(
+            self.command, "--change='/test'", "--backup", "-")
+        assert not result.success, result.stderr
+        assert "applies only when reading from a file" in result.stderr
+
+    def test_bad_data_type_optional(self, script_runner, tmp_path_factory):
+        yaml_file = create_temp_yaml_file(tmp_path_factory, """---
+boolean: false
+""")
+        result = script_runner.run(
+            self.command, "--change=/boolean", "--value=NOT_BOOLEAN",
+            "--format=boolean", yaml_file)
+        assert not result.success, result.stderr
+        assert "Impossible to write 'NOT_BOOLEAN' as boolean." in result.stderr
+
+    def test_bad_data_type_mandatory(self, script_runner, tmp_path_factory):
+        yaml_file = create_temp_yaml_file(tmp_path_factory, """---
+boolean: false
+""")
+        result = script_runner.run(
+            self.command, "--change=/boolean", "--value=NOT_BOOLEAN",
+            "--format=boolean", "--mustexist", yaml_file)
+        assert not result.success, result.stderr
+        assert "Impossible to write 'NOT_BOOLEAN' as boolean." in result.stderr
+
     def test_input_by_value(self, script_runner, tmp_path_factory):
         import re
 
@@ -49,7 +85,8 @@ class Test_yaml_set():
         key: value
         """
         yaml_file = create_temp_yaml_file(tmp_path_factory, content)
-        result = script_runner.run(self.command, "--change=/key", "--value=abc", yaml_file)
+        result = script_runner.run(
+            self.command, "--change=/key", "--value=abc", yaml_file)
         assert result.success, result.stderr
 
         with open(yaml_file, 'r') as fhnd:
@@ -499,3 +536,114 @@ aliases:
         with open(yaml_file, 'r') as fhnd:
             filedat = fhnd.read()
         assert filedat == yamlout
+
+    def test_set_value_in_empty_file(
+        self, script_runner, tmp_path_factory
+    ):
+        yaml_file = create_temp_yaml_file(tmp_path_factory, "")
+        result_content = """---
+some:
+  key:
+    to: nowhere
+"""
+
+        result = script_runner.run(
+            self.command
+            , "--change=some.key.to"
+            , "--value=nowhere"
+            , yaml_file)
+        assert result.success, result.stderr
+
+        with open(yaml_file, 'r') as fhnd:
+            filedat = fhnd.read()
+        assert filedat == result_content
+
+    def test_set_value_in_json_file(
+        self, script_runner, tmp_path_factory
+    ):
+        yaml_file = create_temp_yaml_file(tmp_path_factory, '{"key": "value"}')
+        result_content = '{"key": "changed"}'
+
+        result = script_runner.run(
+            self.command
+            , "--change=key"
+            , "--value=changed"
+            , yaml_file)
+        assert result.success, result.stderr
+
+        with open(yaml_file, 'r') as fhnd:
+            filedat = fhnd.read()
+        assert filedat == result_content
+
+    def test_stdin_to_stdout_yaml(self, script_runner):
+        import subprocess
+
+        stdin_content = """---
+hash:
+  sub_hash:
+    key1: value 1.1
+    key2: value 2.1
+  another_sub_hash:
+    key1: value 1.2
+    key2: value 2.2
+array:
+  - element 1
+  - element 2
+"""
+
+        stdout_content = """---
+hash:
+  sub_hash:
+    key1: CHANGE EVERYTHING!
+    key2: CHANGE EVERYTHING!
+  another_sub_hash:
+    key1: CHANGE EVERYTHING!
+    key2: CHANGE EVERYTHING!
+array:
+  - CHANGE EVERYTHING!
+  - CHANGE EVERYTHING!
+"""
+
+        result = subprocess.run(
+            [self.command
+            , "--change=**"
+            , "--value=CHANGE EVERYTHING!"
+            , "-"]
+            , stdout=subprocess.PIPE
+            , input=stdin_content
+            , universal_newlines=True
+        )
+
+        # DEBUG
+        # print("Expected:")
+        # print(merged_yaml_content)
+        # print("Got:")
+        # print(result.stdout)
+
+        assert 0 == result.returncode, result.stderr
+        assert stdout_content == result.stdout
+
+    def test_stdin_to_stdout_json(self, script_runner):
+        import subprocess
+
+        stdin_content = """{"hash": {"sub_hash": {"key1": "value 1.1", "key2": "value 2.1"}, "another_sub_hash": {"key1": "value 1.2", "key2": "value 2.2"}}, "array": ["element 1", "element 2"]}"""
+
+        stdout_content = """{"hash": {"sub_hash": {"key1": "CHANGE EVERYTHING!", "key2": "CHANGE EVERYTHING!"}, "another_sub_hash": {"key1": "CHANGE EVERYTHING!", "key2": "CHANGE EVERYTHING!"}}, "array": ["CHANGE EVERYTHING!", "CHANGE EVERYTHING!"]}"""
+
+        result = subprocess.run(
+            [self.command
+            , "--change=**"
+            , "--value=CHANGE EVERYTHING!"]
+            , stdout=subprocess.PIPE
+            , input=stdin_content
+            , universal_newlines=True
+        )
+
+        # DEBUG
+        # print("Expected:")
+        # print(merged_yaml_content)
+        # print("Got:")
+        # print(result.stdout)
+
+        assert 0 == result.returncode, result.stderr
+        assert stdout_content == result.stdout
