@@ -11,3 +11,7 @@ class Test_wrappers_NodeCoords():
 	def test_repr(self):
 		node_coord = NodeCoords([], None, None)
 		assert repr(node_coord) == "NodeCoords('[]', 'None', 'None')"
+
+	def test_str(self):
+		node_coord = NodeCoords([], None, None)
+		assert str(node_coord) == "[]"
