@@ -4,20 +4,27 @@ Implement YAML document Merger.
 Copyright 2020 William W. Kimball, Jr. MBA MSIS
 """
 from typing import Any, Dict, List, Set, Tuple
+import json
+from io import StringIO
+from pathlib import Path
 
 import ruamel.yaml # type: ignore
 from ruamel.yaml.scalarstring import ScalarString
 from ruamel.yaml.comments import CommentedSeq, CommentedMap
 
+from yamlpath.func import (
+    append_list_element,
+    build_next_node,
+)
 from yamlpath.wrappers import ConsolePrinter, NodeCoords
 from yamlpath.merger.exceptions import MergeException
 from yamlpath.merger.enums import (
     AnchorConflictResolutions,
     AoHMergeOpts,
     ArrayMergeOpts,
-    HashMergeOpts
+    HashMergeOpts,
+    OutputDocTypes,
 )
-from yamlpath.func import append_list_element
 from yamlpath.merger import MergerConfig
 from yamlpath import YAMLPath, Processor
 
@@ -44,6 +51,36 @@ class Merger:
         self.data: Any = lhs
         self.config: MergerConfig = config
 
+        # ryamel.yaml unfortunately tracks comments AFTER each YAML node.  As
+        # such, it is impossible to copy comments from RHS to LHS in any
+        # sensible way.  Trying leads to absurd merge results that are data-
+        # accurate but comment-insane.  This ruamel.yaml design decision forces
+        # me to simply delete all comments from all merge documents to produce
+        # a sensible result.
+        Merger.delete_all_comments(self.data)
+
+    def _delete_mergeref_keys(self, data: CommentedMap) -> None:
+        """
+        Delete all YAML merge reference keys from a CommentedMap.
+
+        This is necessary when using the insert() method of a CommentedMap
+        because doing so converts all YAML merge references (key-value pairs
+        merged into a YAML Hash via the `<<:` operator) to concrete key-value
+        pairs of the affected Hash.
+        """
+        concrete_keys = []
+        for local_key, _ in data.non_merged_items():
+            concrete_keys.append(local_key)
+
+        reference_keys = set(data.keys()).difference(concrete_keys)
+        for key in reference_keys:
+            self.logger.debug(
+                "Deleting key from LHS:",
+                data=key, prefix="Merger::_delete_mergeref_keys:  ",
+                header="!" * 50
+            )
+            del data[key]
+
     #pylint: disable=too-many-branches
     def _merge_dicts(
         self, lhs: CommentedMap, rhs: CommentedMap, path: YAMLPath
@@ -70,6 +107,20 @@ class Merger:
             raise MergeException(
                 "Impossible to add Hash data to non-Hash destination.", path)
 
+        self.logger.debug(
+            "Merging INTO dict with keys: {}:".format(", ".join(lhs.keys())),
+            data=lhs, prefix="Merger::_merge_dicts:  ",
+            header="--------------------")
+        self.logger.debug(
+            "Merging FROM dict with keys: {}:".format(", ".join(rhs.keys())),
+            data=rhs, prefix="Merger::_merge_dicts:  ",
+            footer="====================")
+
+        # Delete all internal YAML merge reference keys lest any later
+        # .insert() operation on LHS inexplicably convert them from reference
+        # to concrete keys.  This seems like a bug in ruamel.yaml...
+        self._delete_mergeref_keys(lhs)
+
         # Assume deep merge until a node's merge rule indicates otherwise
         buffer: List[Tuple[Any, Any]] = []
         buffer_pos = 0
@@ -78,7 +129,21 @@ class Merger:
             if key in lhs:
                 # Write the buffer if populated
                 for b_key, b_val in buffer:
+                    self.logger.debug(
+                        "Merger::_merge_dicts:  Inserting key, {}, from"
+                        " buffer to position, {}, at path, {}."
+                        .format(b_key, buffer_pos, path_next),
+                        header="INSERT " * 15)
+                    self.logger.debug(
+                        "Before INSERT, the LHS document was:",
+                        data=lhs, prefix="Merger::_merge_dicts:  ")
+                    self.logger.debug(
+                        "... and before INSERT, the incoming value will be:",
+                        data=b_val, prefix="Merger::_merge_dicts:  ")
                     lhs.insert(buffer_pos, b_key, b_val)
+                    self.logger.debug(
+                        "After INSERT, the LHS document became:",
+                        data=lhs, prefix="Merger::_merge_dicts:  ")
                     buffer_pos += 1
                 buffer = []
 
@@ -95,17 +160,41 @@ class Merger:
                 if merge_mode in (HashMergeOpts.LEFT, AoHMergeOpts.LEFT):
                     continue
                 if merge_mode in (HashMergeOpts.RIGHT, AoHMergeOpts.RIGHT):
+                    self.logger.debug(
+                        "Merger::_merge_dicts:  Overwriting key, {}, at path,"
+                        " {}.".format(key, path_next),
+                        header="OVERWRITE " * 15)
                     lhs[key] = val
                     continue
 
                 if isinstance(val, CommentedMap):
                     lhs[key] = self._merge_dicts(lhs[key], val, path_next)
+                    self.logger.debug(
+                        "Document BEFORE calling combine_merge_anchors:",
+                        data=lhs, prefix="Merger::_merge_dicts:  ",
+                        header="+------------------+")
                     Merger.combine_merge_anchors(lhs[key], val)
+                    self.logger.debug(
+                        "Document AFTER calling combine_merge_anchors:",
+                        data=lhs, prefix="Merger::_merge_dicts:  ",
+                        footer="+==================+")
                 elif isinstance(val, CommentedSeq):
                     lhs[key] = self._merge_lists(
                         lhs[key], val, path_next, parent=rhs, parentref=key)
                 else:
+                    self.logger.debug(
+                        "Merger::_merge_dicts:  Updating key, {}, at path,"
+                        " {}.".format(key, path_next), header="UPDATE " * 15)
+                    self.logger.debug(
+                        "Before UPDATE, the LHS document was:",
+                        data=lhs, prefix="Merger::_merge_dicts:  ")
+                    self.logger.debug(
+                        "... and before UPDATE, the incoming value will be:",
+                        data=val, prefix="Merger::_merge_dicts:  ")
                     lhs[key] = val
+                    self.logger.debug(
+                        "After UPDATE, the LHS document became:",
+                        data=lhs, prefix="Merger::_merge_dicts:  ")
             else:
                 # LHS lacks the RHS key.  Buffer this key-value pair in order
                 # to insert it ahead of whatever key(s) follow this one in RHS
@@ -116,8 +205,15 @@ class Merger:
 
         # Write any remaining buffered content to the end of LHS
         for b_key, b_val in buffer:
+            self.logger.debug(
+                "Merger::_merge_dicts:  Appending key, {}, from buffer at"
+                " path, {}.".format(b_key, path), header="APPEND " * 15)
             lhs[b_key] = b_val
 
+        self.logger.debug(
+            "Completed merge result for path, {}:".format(path),
+            data=lhs, prefix="Merger::_merge_dicts:  ")
+
         return lhs
 
     def _merge_simple_lists(
@@ -154,13 +250,11 @@ class Merger:
         for idx, ele in enumerate(rhs):
             path_next = YAMLPath(path).append("[{}]".format(idx))
             self.logger.debug(
-                "Merger::_merge_simple_lists:  Processing element {}{} at {}."
-                .format(ele, type(ele), path_next))
+                "Processing element {} at {}.".format(idx, path_next),
+                prefix="Merger::_merge_simple_lists:  ", data=ele)
 
             if append_all or (ele not in lhs):
-                append_list_element(
-                    lhs, ele,
-                    ele.anchor.value if hasattr(ele, "anchor") else None)
+                lhs.append(ele)
         return lhs
 
     # pylint: disable=locally-disabled,too-many-branches
@@ -197,8 +291,8 @@ class Merger:
                 , path)
 
         self.logger.debug(
-            "Merger::_merge_arrays_of_hashes:  Merging {} Hash(es) at {}."
-            .format(len(rhs), path))
+            "Merging {} Hash(es) at {}.".format(len(rhs), path),
+            prefix="Merger::_merge_arrays_of_hashes:  ", data=rhs)
 
         id_key: str = ""
         if len(rhs) > 0 and isinstance(rhs[0], CommentedMap):
@@ -212,8 +306,8 @@ class Merger:
         for idx, ele in enumerate(rhs):
             path_next = YAMLPath(path).append("[{}]".format(idx))
             self.logger.debug(
-                "Merger::_merge_arrays_of_hashes:  Processing element {} {}"
-                " at {}.".format(ele, type(ele), path_next))
+                "Processing element #{} at {}.".format(idx, path_next),
+                prefix="Merger::_merge_arrays_of_hashes:  ", data=ele)
 
             if merge_mode is AoHMergeOpts.DEEP:
                 if id_key in ele:
@@ -292,8 +386,9 @@ class Merger:
 
         Returns:  (str) The new, unique anchor name.
         """
-        self.logger.debug("Merger::_calc_unique_anchor:  Preexisting Anchors:")
-        self.logger.debug(known_anchors)
+        self.logger.debug(
+            "Preexisting Anchors:", prefix="Merger::_calc_unique_anchor:  ",
+            data=known_anchors)
         aid = 1
         while anchor in known_anchors:
             anchor = "{}_{}".format(anchor, aid)
@@ -311,13 +406,15 @@ class Merger:
         """
         lhs_anchors: Dict[str, Any] = {}
         Merger.scan_for_anchors(self.data, lhs_anchors)
-        self.logger.debug("Merger::_resolve_anchor_conflicts:  LHS Anchors:")
-        self.logger.debug(lhs_anchors)
+        self.logger.debug(
+            "LHS Anchors:", prefix="Merger::_resolve_anchor_conflicts:  ",
+            data=lhs_anchors)
 
         rhs_anchors: Dict[str, Any] = {}
         Merger.scan_for_anchors(rhs, rhs_anchors)
-        self.logger.debug("Merger::_resolve_anchor_conflicts:  RHS Anchors:")
-        self.logger.debug(rhs_anchors)
+        self.logger.debug(
+            "RHS Anchors:", prefix="Merger::_resolve_anchor_conflicts:  ",
+            data=rhs_anchors)
 
         for anchor in [anchor
                 for anchor in rhs_anchors
@@ -332,20 +429,21 @@ class Merger:
             conflict_mode = self.config.anchor_merge_mode()
 
             self.logger.debug(
-                "Merger::_resolve_anchor_conflicts:  Anchor {} is in both"
-                " documents.".format(anchor))
+                "Anchor is in both documents:",
+                prefix="Merger::_resolve_anchor_conflicts:  ", data=anchor)
             self.logger.debug(
-                "Merger::_resolve_anchor_conflicts:  lhs_anchor:")
-            self.logger.debug(lhs_anchor)
+                "lhs_anchor:", prefix="Merger::_resolve_anchor_conflicts:  ",
+                data=lhs_anchor)
             self.logger.debug(
-                "Merger::_resolve_anchor_conflicts:  rhs_anchor:")
-            self.logger.debug(rhs_anchor)
+                "rhs_anchor:", prefix="Merger::_resolve_anchor_conflicts:  ",
+                data=rhs_anchor)
 
             if lhs_anchor != rhs_anchor:
                 if conflict_mode is AnchorConflictResolutions.RENAME:
                     self.logger.debug(
                         "Anchor {} conflict; will RENAME anchors."
-                        .format(anchor))
+                        .format(anchor),
+                        prefix="Merger::_resolve_anchor_conflicts:  ")
                     Merger.rename_anchor(
                         rhs, anchor,
                         self._calc_unique_anchor(
@@ -357,12 +455,14 @@ class Merger:
                 elif conflict_mode is AnchorConflictResolutions.LEFT:
                     self.logger.debug(
                         "Anchor {} conflict; LEFT will override."
-                        .format(anchor))
+                        .format(anchor),
+                        prefix="Merger::_resolve_anchor_conflicts:  ")
                     Merger.replace_anchor(rhs, rhs_anchor, lhs_anchor)
                 elif conflict_mode is AnchorConflictResolutions.RIGHT:
                     self.logger.debug(
                         "Anchor {} conflict; RIGHT will override."
-                        .format(anchor))
+                        .format(anchor),
+                        prefix="Merger::_resolve_anchor_conflicts:  ")
                     Merger.replace_anchor(self.data, lhs_anchor, rhs_anchor)
                 else:
                     raise MergeException(
@@ -394,17 +494,24 @@ class Merger:
         if rhs is None:
             return
 
-        # When LHS is None (empty document), just dump all of RHS into it
-        if self.data is None:
-            self.data = rhs
-            return
-
-        lhs_proc = Processor(self.logger, self.data)
-        insert_at = self.config.get_insertion_point()
-
         # Remove all comments (no sensible way to merge them)
         Merger.delete_all_comments(rhs)
 
+        # When LHS is None (empty document), just dump all of RHS into it,
+        # honoring any --mergeat|-m location as best as possible.
+        insert_at = self.config.get_insertion_point()
+        if self.data is None:
+            self.logger.debug(
+                "Replacing None data with:", prefix="Merger::merge_with:  ",
+                data=rhs, data_header="     *****")
+            self.data = build_next_node(insert_at, 0, rhs)
+            self.logger.debug(
+                "Merged document is now:", prefix="Merger::merge_with:  ",
+                data=self.data, footer="     ***** ***** *****")
+            if isinstance(rhs, (dict, list)):
+                # Only Scalar values need further processing
+                return
+
         # Resolve any anchor conflicts
         self._resolve_anchor_conflicts(rhs)
 
@@ -422,6 +529,7 @@ class Merger:
         # Loop through all insertion points and the elements in RHS
         merge_performed = False
         nodes: List[NodeCoords] = []
+        lhs_proc = Processor(self.logger, self.data)
         for node_coord in lhs_proc.get_nodes(
                 insert_at, default_value=default_val
         ):
@@ -467,11 +575,68 @@ class Merger:
                     lhs_proc.set_value(insert_at, rhs)
                     merge_performed = True
 
+        self.logger.debug(
+            "Completed merge operation, resulting in document:",
+            prefix="Merger::merge_with:  ", data=self.data)
+
         if not merge_performed:
             raise MergeException(
                 "A merge was not performed.  Ensure your target path matches"
                 " at least one node in the left document(s).", insert_at)
 
+    def prepare_for_dump(
+        self, yaml_writer: Any, output_file: str = ""
+    ) -> OutputDocTypes:
+        """
+        Prepare this merged document and its writer for final rendering.
+
+        This coallesces the YAML writer's settings to, in particular,
+        distinguish between YAML and JSON.  It will also force demarcation of
+        every String key and value within the document when the output will be
+        JSON.
+
+        Parameters:
+        1. yaml_writer (ruamel.yaml.YAML) The YAML document writer
+
+        Returns:  (OutputDocTypes) One of:
+          * OutputDocTypes.JSON:  The document and yaml_writer are JSON format.
+          * OutputDocTypes.YAML:  The document and yaml_writer are YAML format.
+        """
+        # Check whether the user is forcing an output format
+        doc_format = self.config.get_document_format()
+        if doc_format is OutputDocTypes.AUTO:
+            # Identify by file-extension, if it indicates a known type
+            file_extension = (Path(output_file).suffix.lower()
+                              if output_file else "")
+            if file_extension in [".json", ".yaml", ".yml"]:
+                is_flow = file_extension == ".json"
+            else:
+                # Check whether the document root is in flow or block format
+                is_flow = True
+                if hasattr(self.data, "fa"):
+                    is_flow = self.data.fa.flow_style()
+        else:
+            is_flow = doc_format is OutputDocTypes.JSON
+
+        if is_flow:
+            # Dump the document as true JSON and reload it; this automatically
+            # exlodes all aliases.
+            xfer_buffer = StringIO()
+            json.dump(self.data, xfer_buffer)
+            xfer_buffer.seek(0)
+            self.data = yaml_writer.load(xfer_buffer)
+
+            # Ensure the writer doesn't emit a YAML Start-of-Document marker
+            yaml_writer.explicit_start = False
+        else:
+            # Ensure block style output
+            Merger.set_flow_style(self.data, False)
+
+            # When writing YAML, ensure the document start mark is emitted
+            yaml_writer.explicit_start = True
+
+        return OutputDocTypes.JSON if is_flow else OutputDocTypes.YAML
+
     @classmethod
     def set_flow_style(cls, node: Any, is_flow: bool) -> None:
         """
@@ -578,17 +743,8 @@ class Merger:
     @classmethod
     def combine_merge_anchors(cls, lhs: CommentedMap, rhs: CommentedMap):
         """Merge YAML merge keys."""
-        if len(rhs.merge) < 1:
-            return
-
-        next_merge_index = -1
-        for mele in lhs.merge:
-            if mele[0] > next_merge_index:
-                next_merge_index = mele[0]
-
         for mele in rhs.merge:
-            next_merge_index += 1
-            lhs.merge.append((next_merge_index, mele[1]))
+            lhs.add_yaml_merge([mele])
 
     @classmethod
     def replace_anchor(
@@ -645,6 +801,9 @@ class Merger:
 
         Returns:  N/A
         """
+        if dom is None:
+            return
+
         if isinstance(dom, CommentedMap):
             for key, val in dom.items():
                 Merger.delete_all_comments(key)
